# -*- Mode: Python -*-
# vi:si:et:sw=4:sts=4:ts=4
#
# groupvte - settings
# Copyright (C) 2005 Michele Baldessari <michele@pupazzo.org>

# This file may be distributed and/or modified under the terms of
# the GNU General Public License version 2 as published by
# the Free Software Foundation.
# This file is distributed without any warranty; without even the implied
# warranty of merchantability or fitness for a particular purpose.
# See "LICENSE.GPL" in the source distribution for more information.

import ConfigParser, logging, os.path, string, traceback
import vte, gtk, gtk.gdk

class ConfigError(Exception):
    pass

SESSIONS = "sessions"
MACHINES = "machines"
MAIN     = "main"

class Settings:
    _MainDefault = {
        'prefix':       '/usr/local',
        'sysconfdir':   '/usr/local/etc',
        'groupvtedir':  '/usr/local/lib/python2.4/site-packages/groupvte',
        'pixmapdir':    '/usr/local/share/groupvte',
        'font':         'Monospace',
        'bell':         True,
        'fontsize':     8,
        'configfile':   os.path.join(os.path.expanduser('~'), '.groupvte.conf'),
        'version':      '0.0.8',
        'pygtkdir':     '/usr/lib/python2.4/site-packages',
        'pygtkversion': '2.6.0',
        'activecolor':  'red',
        'inactivecolor':'black',
        'emulation':    'xterm',
        'maxcolumns':   2
    }

    _ConfigDefault = {
        MAIN: _MainDefault,
        SESSIONS: {},
        MACHINES: {}
    }

    def get_version(self):
        if self.config.has_key(MAIN) and \
            self.config[MAIN].has_key('version'):
            return self.config[MAIN]['version']
        return None

    def get_pixmapdir(self):
        if self.config.has_key(MAIN) and \
            self.config[MAIN].has_key('pixmapdir'):
            return self.config[MAIN]['pixmapdir']
        return None

    def get_active_color(self):
        if self.config.has_key(MAIN) and \
            self.config[MAIN].has_key('activecolor'):
            return self.config[MAIN]['activecolor']
        return None

    def get_inactive_color(self):
        if self.config.has_key(MAIN) and \
            self.config[MAIN].has_key('inactivecolor'):
            return self.config[MAIN]['inactivecolor']
        return None
    
    def get_max_columns(self):
        if self.config.has_key(MAIN) and \
            self.config[MAIN].has_key('maxcolumns'):
            return int(self.config[MAIN]['maxcolumns'])
        return None

    def set_new_path(self, bin_abspath):
        if not self.config.has_key(MAIN):
            return
        path = os.path.normpath(os.path.abspath(bin_abspath))
        (basepath, basename) = os.path.split(path)
        self.config[MAIN]['prefix'] = basepath
        self.config[MAIN]['pyexecdir'] = basepath
        (prebasepath, tmp) = os.path.split(basepath)
        self.config[MAIN]['pixmapdir'] = prebasepath

    def get_session_list(self):
        list = []
        for i in self.config[SESSIONS].keys():
                list.append(i)
        return list

    def get_session_hash(self):
        if self.config.has_key(SESSIONS):
            return self.config[SESSIONS]
        return None

    def has_session(self, session):
        if self.config.has_key(SESSIONS):
            return self.config[SESSIONS].has_key(session)
        return None


    def has_machine(self, machine):
        if self.config.has_key(MACHINES):
            return self.config[MACHINES].has_key(machine)
        return None
    
    def get_machine_list(self):
        list = []
        for i in self.config[MACHINES].keys():
                list.append(i)
        return list

    def get_machine_hash(self):
        if self.config.has_key(MACHINES):
            return self.config[MACHINES]
        return None

    def get_machine(self, machine):
        if self.config.has_key(MACHINES) and \
            self.config[MACHINES].has_key(machine):
            return self.config[MACHINES][machine]
        return None

    def get_session(self, session):
        if self.config.has_key(SESSIONS) and \
            self.config[SESSIONS].has_key(session):
            return self.config[SESSIONS][session]
        return None

    def get_font_and_size(self):
        if self.config.has_key(MAIN) and \
            self.config[MAIN].has_key("font"):
            return (self.config[MAIN]["font"], self.config[MAIN]["fontsize"])
        return None

    def set_font_and_size(self, font, fontsize):
        if self.config.has_key(MAIN):
            self.config[MAIN]["font"] = font
            self.config[MAIN]["fontsize"] = fontsize
        else:
            raise ConfigError, "No [main] section present"

    def get_color(self):
        if self.config.has_key(MAIN) and \
            self.config[MAIN].has_key("color"):
            return self.config[MAIN]["color"]
        return None

    def set_color(self, color):
        if self.config.has_key(MAIN):
            self.config[MAIN]["color"] = color
        else:
            raise ConfigError, "No [main] section present"

    def get_bell(self):
        if self.config.has_key(MAIN) and \
            self.config[MAIN].has_key("bell"):
            return self.config[MAIN]["bell"]
        return None

    def set_bell(self, bell):
        if self.config.has_key(MAIN):
            self.config[MAIN]["bell"] = bell
        else:
            raise ConfigError, "No [main] section present"
    
    def get_default_session(self):
        if self.config.has_key(MAIN) and \
            self.config[MAIN].has_key("defaultsession"):
            return self.config[MAIN]["defaultsession"]
        return None
    
    def set_default_session(self, session):
        if self.config.has_key(MAIN):
            self.config[MAIN]["defaultsession"] = session
        else:
            raise ConfigError, "No [main] section present"
    
    def get_command(self, machine):
        if self.config.has_key(MACHINES) and \
            self.config[MACHINES].has_key(machine) and \
            self.config[MACHINES][machine].has_key("command"):
            c = self.config[MACHINES][machine]["command"]
            if not isinstance(c, list):
                c = (c, )
            return c
        return None

    def get_machine_name(self, machine):
        if self.config.has_key(MACHINES) and \
            self.config[MACHINES].has_key(machine) and \
            self.config[MACHINES][machine].has_key("name"):
            return self.config[MACHINES][machine]["name"]
        return None

    def add_machine(self, machine, commands):
        if not self.config.has_key(MACHINES):
            raise ConfigError, "No [machines] section present"
        
        if self.config[MACHINES].has_key(machine):
            raise ConfigError, "Machine %s already exists" % (machine)

        self.config[MACHINES][machine]["command"] = commands

    def change_machine(self, machine, commands):
        if not self.config.has_key(MACHINES):
            raise ConfigError, "No [machines] section present"
        
        if not self.config[MACHINES].has_key(machine):
            raise ConfigError, "Machine %s should already exist" % (machine)

        self.config[MACHINES][machine]["command"] = commands

    def remove_machine(self, machine):
        if not self.config.has_key(MACHINES):
            raise ConfigError, "No [machines] section present"
        if not self.config[MACHINES].has_key(machine):
            raise ConfigError, "Machine %s can't be found" % (machine)
        
        del self.config[MACHINES][machine]

    def add_session(self, session, machinelist):
        if not self.config.has_key(SESSIONS):
            raise ConfigError, "No [session] section present"
        
        if self.config[SESSIONS].has_key(session):
            raise ConfigError, "Session %s already exists" % (session)

        self.config[SESSIONS][session] = machinelist

    def change_session(self, session, machinelist):
        if not self.config.has_key(SESSIONS):
            raise ConfigError, "No [sessions] section present"
        
        if not self.config[SESSIONS].has_key(session):
            raise ConfigError, "Session %s should already exist" % (session)

        self.config[SESSIONS][session] = machinelist
        
    def remove_session(self, session):
        if not self.config.has_key(SESSIONS):
            raise ConfigError, "No [sessions] section present"
        if not self.config[SESSIONS].has_key(session):
            raise ConfigError, "Session %s can't be found" % (session)
        
        del self.config[SESSIONS][session]

    def parse(self):
        config = self._ConfigDefault.copy()
        cp = ConfigParser.ConfigParser()
        try:
            cp.read(self.conffile)
            for j in cp.options(MAIN):
                config[MAIN][j] = string.strip(cp.get(MAIN, j))
            for j in cp.options(SESSIONS):
                config[SESSIONS][j] = string.strip(cp.get(SESSIONS, j)).split(",")
            for i in cp.sections():
                name = i.lower()
                if name in [MAIN, SESSIONS]:
                    continue
                d = {}
                for j in cp.options(i):
                    d[string.lower(j)] = string.strip(cp.get(i, j))
                    config[MACHINES][i] = d
        except:
            traceback.print_exc()
            raise ConfigError, "Error in configuration file %s" % (self.conffile)


        self.config = config

        m = self.get_machine_list()
        for i in self.get_session_list():
            s = self.get_session(i)
            for j in s:
                if not j in m:
                    raise ConfigError, "Session %s has the non existant machine %s" % (i, j)

        for i in m:
            j = self.get_machine(i)
            if not j.has_key("name"):
                raise ConfigError, "Machine %s lacks name" % (i)
            if not j.has_key("command"):
                raise ConfigError, "Machine %s lacks command" % (i)

    def write(self, conffile=""):
        # FIXME: LOCK FILE 
        if conffile != "":
            self.conffile = conffile
        f = open(self.conffile, "w")
        for i in [MAIN,SESSIONS]:
            f.write("[%s]\n" % (i))
            tmp = self.config[i]
            for j in tmp.keys():
                if isinstance(tmp[j], list):
                    f.write("%s = %s\n" % (j, ",".join(tmp[j])))
                else:
                    f.write("%s = %s\n" % (j, tmp[j]))

        machines = self.config[MACHINES]
        for i in machines.keys():
            f.write("[%s]\n" % (i))
            for j in machines[i].keys():
                if isinstance(machines[i][j], list):
                    f.write("%s = %s\n" % (i, ",".join(machines[i][j])))
                else:
                    f.write("%s = %s\n" % (j, machines[i][j]))
        f.close()

    def __init__(self, conffile=""):
        if conffile != "":
            self.conffile = conffile
        else:
            self.conffile = self._ConfigDefault[MAIN]['configfile']
        self.parse()
